@@ -1,6 +1,9 @@
 package ZMQ::FFI::SocketRole;
-$ZMQ::FFI::SocketRole::VERSION = '1.00';
+$ZMQ::FFI::SocketRole::VERSION = '1.05_01';
 use FFI::Platypus;
+use FFI::Platypus::Memory qw(malloc);
+
+use ZMQ::FFI::Constants qw(zmq_msg_t_size);
 
 use Moo::Role;
 
@@ -28,12 +31,39 @@ has _socket => (
     default => -1,
 );
 
+# message struct to reuse when sending/receiving
+has _zmq_msg_t => (
+    is        => 'ro',
+    lazy      => 1,
+    builder   => '_build_zmq_msg_t',
+);
+
+# used to make sure we handle fork situations correctly
+has _pid => (
+    is      => 'ro',
+    default => sub { $$ },
+);
+
 has sockopt_sizes => (
     is      => 'ro',
     lazy    => 1,
     builder => '_build_sockopt_sizes'
 );
 
+sub _build_zmq_msg_t {
+    my ($self) = @_;
+
+    my $msg_ptr;
+    {
+        no strict q/refs/;
+        my $class = ref $self;
+        $msg_ptr = malloc(zmq_msg_t_size);
+        &{"$class\::zmq_msg_init"}($msg_ptr);
+    }
+
+    return $msg_ptr;
+}
+
 sub _build_sockopt_sizes {
     my $ffi = FFI::Platypus->new();
 
@@ -81,7 +111,7 @@ ZMQ::FFI::SocketRole
 
 =head1 VERSION
 
-version 1.00
+version 1.05_01
 
 =head1 AUTHOR
 
