@@ -1,5 +1,5 @@
 package ZMQ::FFI::ZMQ2::Context;
-$ZMQ::FFI::ZMQ2::Context::VERSION = '1.00';
+$ZMQ::FFI::ZMQ2::Context::VERSION = '1.05_01';
 use FFI::Platypus;
 use ZMQ::FFI::Util qw(zmq_soname);
 use ZMQ::FFI::Constants qw(ZMQ_STREAMER);
@@ -11,7 +11,7 @@ use namespace::clean;
 
 with qw(
     ZMQ::FFI::ContextRole
-    ZMQ::FFI::ErrorHandler
+    ZMQ::FFI::ErrorHelper
     ZMQ::FFI::Versioner
 );
 
@@ -66,6 +66,16 @@ sub _load_zmq2_ffi {
         # int zmq_term(void *context)
         'zmq_term' => ['pointer'] => 'int'
     );
+
+    $ffi->attach(
+        # const char *zmq_strerror(int errnum)
+        'zmq_strerror' => ['int'] => 'string'
+    );
+
+    $ffi->attach(
+        # int zmq_errno(void)
+        'zmq_errno' => [] => 'int'
+    );
 }
 
 sub get {
@@ -93,7 +103,6 @@ sub socket {
         ctx          => $self,
         type         => $type,
         soname       => $self->soname,
-        error_helper => $self->error_helper,
     );
 }
 
@@ -127,6 +136,9 @@ sub device {
 sub destroy {
     my ($self) = @_;
 
+    # don't try to cleanup context cloned from another process (fork)
+    return unless $self->_pid == $$;
+
     $self->check_error(
         'zmq_term',
         zmq_term($self->_ctx)
@@ -157,7 +169,7 @@ ZMQ::FFI::ZMQ2::Context
 
 =head1 VERSION
 
-version 1.00
+version 1.05_01
 
 =head1 AUTHOR
 
