@@ -1,5 +1,5 @@
 package ZMQ::FFI::ZMQ3::Context;
-$ZMQ::FFI::ZMQ3::Context::VERSION = '1.00';
+$ZMQ::FFI::ZMQ3::Context::VERSION = '1.05_01';
 use FFI::Platypus;
 use ZMQ::FFI::Util qw(zmq_soname);
 use ZMQ::FFI::Constants qw(ZMQ_IO_THREADS ZMQ_MAX_SOCKETS);
@@ -11,7 +11,7 @@ use namespace::clean;
 
 with qw(
     ZMQ::FFI::ContextRole
-    ZMQ::FFI::ErrorHandler
+    ZMQ::FFI::ErrorHelper
     ZMQ::FFI::Versioner
 );
 
@@ -77,6 +77,16 @@ sub _load_zmq3_ffi {
         # int zmq_ctx_destroy (void *context)
         'zmq_ctx_destroy' => ['pointer'] => 'int'
     );
+
+    $ffi->attach(
+        # const char *zmq_strerror(int errnum)
+        'zmq_strerror' => ['int'] => 'string'
+    );
+
+    $ffi->attach(
+        # int zmq_errno(void)
+        'zmq_errno' => [] => 'int'
+    );
 }
 
 sub get {
@@ -104,7 +114,6 @@ sub socket {
         ctx          => $self,
         type         => $type,
         soname       => $self->soname,
-        error_helper => $self->error_helper,
     );
 }
 
@@ -133,6 +142,9 @@ sub device {
 sub destroy {
     my ($self) = @_;
 
+    # don't try to cleanup context cloned from another process (fork)
+    return unless $self->_pid == $$;
+
     $self->check_error(
         'zmq_ctx_destroy',
         zmq_ctx_destroy($self->_ctx)
@@ -163,7 +175,7 @@ ZMQ::FFI::ZMQ3::Context
 
 =head1 VERSION
 
-version 1.00
+version 1.05_01
 
 =head1 AUTHOR
 
